"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const aws_sdk_1 = __importDefault(require("aws-sdk"));
const crng_1 = require("crng");
const moment = require("moment");
const assertValidKey = (key) => {
    if (typeof key != 'string' || key[0] != '/') {
        throw new SyntaxError(`CloudFront keys must be a string and start with a forward slash (got "${key}")`);
    }
};
class CloudFront {
    constructor(settings) {
        this.distributionID = settings.distributionID;
        this.cf = new aws_sdk_1.default.CloudFront({
            accessKeyId: settings.accessKeyID,
            secretAccessKey: settings.secretAccessKey,
        });
    }
    invalidate(...keys) {
        return __awaiter(this, void 0, void 0, function* () {
            keys.forEach(key => assertValidKey(key));
            const timestamp = moment().format();
            const randomSuffix = crng_1.cryptoRandomHex(3);
            yield this.cf.createInvalidation({
                DistributionId: this.distributionID,
                InvalidationBatch: {
                    CallerReference: `shasync_cf_${this.distributionID}_invalidation_${timestamp}_${randomSuffix}`,
                    Paths: {
                        Quantity: keys.length,
                        Items: keys,
                    },
                },
            }).promise();
        });
    }
}
exports.CloudFront = CloudFront;
