"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const request = require("request");
const assertValidKey = (key) => {
    if (typeof key != 'string' || key[0] != '/') {
        throw new SyntaxError(`Cloudflare keys must be a string and start with a forward slash (got "${key}")`);
    }
};
class Cloudflare {
    constructor(settings) {
        this.zoneID = settings.zoneID;
        this.site = settings.site;
        this.email = settings.email;
        this.globalAPIKey = settings.globalAPIKey;
    }
    invalidate(...keys) {
        return __awaiter(this, void 0, void 0, function* () {
            keys.forEach(key => assertValidKey(key));
            let batches = [];
            do {
                batches.push(keys.splice(0, 500));
            } while (keys.length);
            yield Promise.all(batches.map(b => this._invalidateBatch(b)));
        });
    }
    _invalidateBatch(keys) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                request({
                    method: 'DELETE',
                    url: `https://api.cloudflare.com/client/v4/zones/${this.zoneID}/purge_cache`,
                    headers: {
                        'X-Auth-Email': this.email,
                        'X-Auth-Key': this.globalAPIKey,
                    },
                    body: {
                        files: keys.map(key => `${this.site}${key}`),
                    },
                    json: true,
                }, (err, _, body) => {
                    if (err) {
                        reject(err);
                        return;
                    }
                    if (body.errors) {
                        body.errors.forEach((err) => console.error(`Error ${err.code}: ${err.message}`));
                    }
                    if (body.messages) {
                        body.messages.forEach((msg) => console.info(msg));
                    }
                    if (body.success) {
                        keys.forEach(key => {
                            console.log(`Invalidated ${key}`);
                        });
                        resolve();
                    }
                    else {
                        reject(new Error('Cloudflare API call not successful'));
                    }
                });
            });
        });
    }
}
exports.Cloudflare = Cloudflare;
