"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const sacli = __importStar(require("sacli"));
const Cloudflare_1 = require("./CDNService/Cloudflare");
const CloudFront_1 = require("./CDNService/CloudFront");
const S3_1 = require("./StorageService/S3");
const diff_1 = require("./Sync/diff");
const sync_1 = require("./Sync/sync");
const getEnv = (name) => {
    const val = process.env[name];
    if (!val) {
        throw new ReferenceError(`Environment variable "${name}" not set`);
    }
    return val;
};
exports.CLI = sacli.build({
    name: 'shasync',
    commands: [
        {
            name: '',
            description: 'Syncronise local files with a cloud file storage provider. Can also invalidate changed remote files with a CDN.',
            options: [
                {
                    alias: 'd',
                    name: 'directory',
                    description: 'Path to local folder to synchronise',
                    type: String,
                    typeLabel: '<dir>',
                },
                {
                    alias: 'p',
                    name: 'prefix',
                    description: 'Prefix of cloud storage object keys',
                    type: String,
                    typeLabel: '<prefix>',
                },
                {
                    alias: 's',
                    name: 'storage',
                    description: 'Cloud storage service; one of `aws` or `b2`; provide AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY, and AWS_REGION as environment variables if `aws`',
                    type: String,
                    typeLabel: 'aws|b2',
                },
                {
                    alias: 'b',
                    name: 'bucket',
                    description: 'Cloud storage bucket',
                    type: String,
                    typeLabel: '<bucket>',
                },
                {
                    alias: 'c',
                    name: 'cdn',
                    description: 'CDN service; one of `aws` or `cf`; provide AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY, and AWS_DISTRIBUTION as environment variables if `aws`; provide CF_ZONE_ID, CF_SITE, CF_EMAIL, CF_API_KEY as environment variables if `cf`',
                    type: String,
                    typeLabel: 'aws|cf',
                },
                {
                    alias: 'i',
                    name: 'index',
                    description: '[Optional] Directory index file name to invalidate when creating or deleting cloud directories',
                    type: String,
                    typeLabel: '<index>',
                },
            ],
            action: ({ bucket, cdn, directory, index, prefix, storage, }) => __awaiter(void 0, void 0, void 0, function* () {
                let storageService;
                switch (storage) {
                    case 'aws':
                        storageService = new S3_1.S3({
                            accessKeyID: getEnv('AWS_ACCESS_KEY_ID'),
                            secretAccessKey: getEnv('SECRET_ACCESS_KEY'),
                            bucket,
                            region: getEnv('AWS_REGION'),
                        });
                        break;
                    case 'b2':
                        // TODO
                        throw new Error(`B2 is currently not supported`);
                    default:
                        throw new TypeError('Unrecognised cloud storage service');
                }
                let cdnService;
                switch (cdn) {
                    case 'aws':
                        cdnService = new CloudFront_1.CloudFront({
                            accessKeyID: getEnv('AWS_ACCESS_KEY_ID'),
                            secretAccessKey: getEnv('SECRET_ACCESS_KEY'),
                            distributionID: getEnv('AWS_DISTRIBUTION'),
                        });
                        break;
                    case 'cf':
                        cdnService = new Cloudflare_1.Cloudflare({
                            email: getEnv('CF_EMAIL'),
                            globalAPIKey: getEnv('CF_API_KEY'),
                            site: getEnv('CF_SITE'),
                            zoneID: getEnv('CF_ZONE_ID'),
                        });
                        break;
                    default:
                        throw new TypeError('Unrecognised CDN service');
                }
                const d = yield diff_1.diff({
                    directory,
                    prefix,
                    storageService,
                    directoryIndex: index,
                });
                yield sync_1.sync({
                    diff: d,
                    directory,
                    prefix,
                    storageService,
                    CDNService: cdnService,
                });
            }),
        },
    ],
});
