"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const Path = __importStar(require("path"));
const common_1 = require("./common");
const getLocalHashes_1 = require("./getLocalHashes");
const getRemoteHashes_1 = require("./getRemoteHashes");
const assertValidPrefix = (prefix) => {
    // Prefixes must not start or end with a slash, and cannot contain backward slashes, but can contain forward slashes, and can be an empty string.
    if (!/^(?:[^\/\\]+\/)*[^\/\\]*$/.test(prefix)) {
        throw new TypeError(`Invalid prefix "${prefix}"`);
    }
};
// TODO Refactor: this function is hard to test and awkwardly returns localHashes.
exports.diff = ({ directory, prefix = '', storageService, directoryIndex, }) => __awaiter(void 0, void 0, void 0, function* () {
    assertValidPrefix(prefix);
    const localHashes = yield getLocalHashes_1.getLocalHashes({ directory });
    const remoteHashes = yield getRemoteHashes_1.getRemoteHashes({ prefix, storageService });
    // Path.relative will convert / to \ on Windows systems, even if base or path has /
    const remoteFiles = new Set([...remoteHashes.keys()].map(k => common_1.normaliseFilePath(Path.relative(prefix, k))));
    // Paths of all the files stored locally (folders not included) e.g. [ index.html, a/b.js, ... ]
    const localFiles = new Set(localHashes.keys());
    const extraRemote = [];
    const missingRemote = [];
    const differentRemote = [];
    const changedPaths = [];
    const invalidateFile = (key) => {
        const path = `/${prefix}/${key}`;
        changedPaths.push(path);
        if (directoryIndex && path.endsWith(`/${directoryIndex}`)) {
            const dirname = Path.dirname(path);
            changedPaths.push(dirname);
            changedPaths.push(`${dirname}/`);
        }
    };
    for (const rf of remoteFiles) {
        if (!localFiles.has(rf)) {
            extraRemote.push(rf);
            invalidateFile(rf);
        }
    }
    for (const lf of localFiles) {
        if (!remoteFiles.has(lf)) {
            missingRemote.push(lf);
            invalidateFile(lf);
        }
        else {
            if (localHashes.get(lf) !== remoteHashes.get(lf)) {
                differentRemote.push(lf);
                invalidateFile(lf);
            }
        }
    }
    return { extraRemote, missingRemote, differentRemote, changedPaths, localHashes };
});
