"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const hashStream_1 = require("../Utils/hashStream");
const common_1 = require("./common");
exports.getRemoteHashes = ({ prefix, storageService, }) => __awaiter(void 0, void 0, void 0, function* () {
    // Paths of all the files remotely
    // Get all object keys, including ones that end with a slash (they should not be there)
    const remoteFiles = yield storageService.list({ prefix })
        .then(list => list.map(o => o.key)); // [ music/index.html, music/lib/ooml.js, ... ]
    const remoteFileHashes = new Map();
    const remoteFilesWithoutHashes = yield Promise.all(remoteFiles.map(k => storageService.getMetadata(k)))
        .then(data => data.filter(({ key, metadata }) => {
        const remoteHash = metadata[common_1.REMOTE_FILE_METADATA_KEY_HASH];
        if (remoteHash) {
            // If cloud file already has hash, get hash
            remoteFileHashes.set(path.relative(prefix, key), remoteHash);
        }
        return !remoteHash;
    }));
    const hashesForRemoteFiles = yield Promise.all(remoteFilesWithoutHashes.map(o => {
        console.log(`Remote file "${o.key}" does not have a hash, hashing...`);
        return hashStream_1.hashStream(common_1.HASH_TYPE, storageService.stream(o.key));
    })).then(data => data.map((hash, i) => {
        const key = remoteFilesWithoutHashes[i].key;
        return { key, hash };
    }));
    yield Promise.all(hashesForRemoteFiles.map(({ key, hash }) => storageService.setMetadata({
        key: key,
        metadata: {
            [common_1.REMOTE_FILE_METADATA_KEY_HASH]: hash,
        },
    })))
        .then(data => data.forEach((_, i) => {
        const { key, hash } = hashesForRemoteFiles[i];
        console.log(`Hashed remote file "${key}": ${hash}`);
        remoteFileHashes.set(path.relative(prefix, key), hash);
    }));
    return remoteFileHashes;
});
