"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const mime_types_1 = require("mime-types");
const path = __importStar(require("path"));
const ask_1 = require("../Utils/ask");
const common_1 = require("./common");
exports.sync = ({ directory, prefix = '', storageService, CDNService, diff, }) => __awaiter(void 0, void 0, void 0, function* () {
    const { extraRemote, missingRemote, differentRemote, changedPaths, localHashes } = diff;
    if (!extraRemote.length) {
        console.log('\nNo extra files');
    }
    else {
        console.log(`\nExtra files found remotely to delete (${extraRemote.length}):\n`);
        console.log(extraRemote.join('\n'));
    }
    if (!missingRemote.length) {
        console.log('\nNo missing files');
    }
    else {
        console.log(`\nMissing files not found remotely to upload (${missingRemote.length}):\n`);
        console.log(missingRemote.join('\n'));
    }
    if (!differentRemote.length) {
        console.log('\nNo different files');
    }
    else {
        console.log(`\nFiles different remotely to upload (${differentRemote.length}):\n`);
        console.log(differentRemote.join('\n'));
    }
    if (!extraRemote.length && !missingRemote.length && !differentRemote.length) {
        console.log('\nNothing to do, sync complete');
        // WARNING: Function exists here.
        return;
    }
    const confirm = yield ask_1.ask('\nAre you sure you want to sync? (y) ');
    if (confirm.toLowerCase() !== 'y') {
        throw new Error(`Sync cancelled by user`);
    }
    if (extraRemote.length) {
        console.log('\n=============== DELETING FILES ===============\n');
        yield storageService.delete(...extraRemote.map(f => `${prefix}/${f}`));
    }
    if (missingRemote.length) {
        console.log('\n=============== UPLOADING FILES ===============\n');
        yield Promise.all(missingRemote.concat(differentRemote).map(f => {
            return storageService.upload({
                key: `${prefix}/${f}`,
                data: fs.createReadStream(path.join(directory, f)),
                contentType: mime_types_1.contentType(path.extname(f)) || 'application/octet-stream',
                metadata: {
                    [common_1.REMOTE_FILE_METADATA_KEY_HASH]: localHashes.get(f),
                },
            });
        }));
    }
    if (changedPaths.length) {
        console.log('\n============== INVALIDATING FILES =============\n');
        yield CDNService.invalidate(...changedPaths);
    }
});
